@@ -12,6 +12,8 @@ from starlette.applications import Starlette
 from starlette.responses import JSONResponse
 from starlette.routing import Route
 from starlette.exceptions import HTTPException
+from starlette.middleware import Middleware
+from starlette.middleware.base import BaseHTTPMiddleware
 from functools import partial
 import asyncio
 from hypercorn.config import Config
@@ -41,6 +43,14 @@ status_map = {
 }
 
 
+class AccessLogMiddleware(BaseHTTPMiddleware):
+  async def dispatch(self, request, call_next):
+    response = await call_next(request)
+    if response.status_code < 400:
+      logging.info(f'[{response.status_code}] {request.method} {request.url}')
+    return response
+
+
 class GatewayServer(threading.Thread):
   handlers = {}
   client = None
@@ -73,6 +83,8 @@ class GatewayServer(threading.Thread):
       raise HTTPException(http_status_code, ex._state.details)
 
   async def http_exception(self, request, exc):
+    if 'favicon.ico' not in str(request.url):
+      logging.error(f'[{exc.status_code}] {request.method} {request.url} - {exc.detail}')
     return JSONResponse({'detail': exc.detail, 'status': exc.status_code}, status_code=exc.status_code)
 
   def run(self) -> None:
@@ -99,8 +111,11 @@ class GatewayServer(threading.Thread):
                 route = Route(http_path, endpoint=self.request_handler, methods=[supported_method])
                 routes.append(route)
                 self.handlers[http_path] = {'service': service_handler._name, 'method': method_name}
-
-    app = Starlette(debug=True, routes=routes, exception_handlers={HTTPException: self.http_exception})
+    middleware = [Middleware(AccessLogMiddleware)]
+    app = Starlette(debug=True,
+                    routes=routes,
+                    exception_handlers={HTTPException: self.http_exception},
+                    middleware=middleware)
     config = Config()
     config.bind = [f'0.0.0.0:{self.port}']
     config.loglevel = 'INFO'
